"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
const events_1 = require("events");
const nats_1 = require("nats");
const seedrandom_1 = __importDefault(require("seedrandom"));
class KeyvNats extends events_1.EventEmitter {
    constructor(options) {
        super();
        this.ttlSupport = true;
        this.client = undefined;
        this.opts = { servers: 'localhost:4222' };
        this.jetStream = undefined;
        this.namespace = 'keyv';
        /**
         * Time to live in milliseconds
         * @private ttl
         */
        this.ttl = 0;
        this.opts = options || this.opts;
        this.namespace = this.opts.namespace || this.namespace;
        this.ttl = this.opts.ttl || this.ttl;
    }
    async connect() {
        try {
            this.client = await (0, nats_1.connect)(this.opts);
            this.jetStream = this.client.jetstream();
            await this.createBucket();
        }
        catch (error) {
            this.emit('error', error);
        }
        return this.client;
    }
    async createBucket() {
        if (this.jetStream)
            this.keyValueBucket = await this.jetStream.views.kv(this.namespace, {
                ttl: this.ttl
            });
    }
    async disconnect() {
        return this.client !== undefined ? await this.client.close() : false;
    }
    /**
     * Deletes the bucket and recreates it
     */
    clear() {
        if (this.keyValueBucket)
            return this.keyValueBucket.destroy().then(() => {
                this.createBucket().then(() => {
                    return;
                });
            });
    }
    delete(key) {
        if (this.keyValueBucket)
            return this.keyValueBucket.delete(getRandomisedKey(key)).then(() => {
                return true;
            });
        return false;
    }
    get(key) {
        if (this.keyValueBucket)
            // @ts-expect-error - Value needs to be number, string or buffer
            return this.keyValueBucket.get(getRandomisedKey(key)).then((kvValue) => {
                if (kvValue) {
                    const value = (0, nats_1.StringCodec)().decode(kvValue.value);
                    return value ? value : undefined;
                }
            }).catch((reason) => {
                // catch get for the clear all case, when bucket is not re-created yet
            });
        return undefined;
    }
    set(key, value) {
        if (this.keyValueBucket)
            // @ts-expect-error - Value needs to be number, string or buffer
            return this.keyValueBucket.put(getRandomisedKey(key), (0, nats_1.StringCodec)().encode(value));
        return undefined;
    }
}
/**
 * Key's can be complex jsons. This function generates a random but unique int 32 value for the given json.
 * This makes the key valid to store in the NATS's kv bucket.
 */
const getRandomisedKey = (key) => {
    return (0, seedrandom_1.default)(key.replace(":", "_")).int32().toString();
};
module.exports = KeyvNats;
//# sourceMappingURL=index.js.map